<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Repositories\GpeEsocialRegistro;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1300 - Pagamentos Diversos na versão 1.1.
 * Classe provedora do datasource utilizado para a geracao do evento S-1300 - Contribuição Sindical Patronal na versão 2.0.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S1300
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1300, Pagamentos Diversos na versão 1.0.
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1300, Contribuição Sindical Patronal na versão 2.0.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $idRetificacao           Identificador de Retificação dos arquivos ou não.
     * @param $nrRecibo                Número do Recibo do primeiro envio em caso de Retificação.
     * @param $idApuracao              Identificador da apuração Mensal ou Anual.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo, $indApuracao)
    {
        if ($versaoEsocial === '2.0') {
            $S1300 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S1300_v20(
                $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo, $indApuracao
            );
        }
        else {
            $nrGrupEsoc = '10';

            $S1300 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S1300_v11(
                $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $indRetificacao, $nrRecibo, $indApuracao, $nrGrupEsoc
            );

            foreach ($S1300 as &$row) {
                //Pagamento à pessoa residente ou domiciliada no Brasil
                if ($row['CDPAIS'] == null) {
                    $pagtos = GpeEsocialRegistro::s1300_Pagamentos($row['NRVINCULOM'], $row['NRGRPITEM'], $row['NRTIPOMOVIMENT'], $nrGrupEsoc, $inicioPeriodo, $fimPeriodo, $nrorg);

                    //Beneficiário Pessoa Física
                    if ($row['CDTIPOINSCRICAO'] == 'CPF') {
                        $contPagto = 1;
                        foreach ($pagtos as &$pagto) {
                            $detalhes = GpeEsocialRegistro::s1300_detPagtoBPF($row[NRVINCULOM], $pagto[DTOCORRENCIA], $row[NRGRPITEM], $row[NRTIPOMOVIMENT], $nrGrupEsoc, $inicioPeriodo, $fimPeriodo, $nrorg);

                            $contDetalhe = 1;
                            $detCompetencia = null;
                            foreach ($detalhes as &$detalhe) {
                                $detCompetencia[$contDetalhe] =
                                    array("_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_DETCOMPETENCIA" => '1',
                                        "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_DETCOMPETENCIA_INDPERREFERENCIA" => $detalhe['INDPERREFERENCIA'],
                                        "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_DETCOMPETENCIA_PERREFPAGTO" => $detalhe['PERREFPAGTO'],
                                        "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_DETCOMPETENCIA_VLRRENDTRIBUTAVEL" => $detalhe['VLRRENDTRIBUTAVEL']);
                                $contDetalhe++;
                            }

                            $pagtoBPF[$contPagto] =
                                array("_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF" => '1',
                                    "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_DTPAGTO" => $pagto['DTPAGTO'],
                                    "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_INDSUSPEXIGIBILIDADE" => $pagto['INDSUSPEXIGIBILIDADE'],
                                    "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_INDDECTERCEIRO" => $pagto['INDDECTERCEIRO'],
                                    "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_VLRRENDTRIBUTAVEL" => $pagto['VLRRENDTRIBUTAVEL'],
                                    "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_VLRIRRF" => $pagto['VLRIRRF'],
                                    "_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF_DETCOMPETENCIA" => $detCompetencia);

                            $pagtoResidente = array("_EVTPAGTOSDIVERSOS_INFOPAGTO_PAGTORESIDENTE_PAGTOBPF" => $pagtoBPF);

                            $row['PAGTORESIDENTE'][1] = $pagtoResidente;

                            $contPagto++;
                        }
                    } //Beneficiário Pessoa Júridica
                    else {
                        //@todo - Desenvolver rotina para gerar valores do pagamento para pessoa jurídica
                    }
                } /*Pagamento à não residente no Brasil ou domiciliado no Exterior*/
                else {
                    //@todo - Desenvolver rotina para gerar valores do pagamento para não residentes no Brasil
                }
            }
        }

        return $S1300;
    }

}