﻿<?php
/**
 * GepAtividadeLogic.
 *
 * @package Logic
 * @author  samuelleite <username@example.com>
 */
class GepAtividadeLogic{


	private $_GepConvocadoreuniaoEntity;

	public function __construct(){
		$this->_GepConvocadoreuniaoEntity = new GepConvocadoreuniaoEntity();
	}


    /**
     * Gera o novo c?o de Atividade.
     *
     * @param integer $nrorg Numero da organizacao.
     *
     * @return Ambiguous
     */
    public static function retornaProxNumAtividade($nrorg){
        $codigo = GepAtividadeEntity::retornaProxNumAtividade($nrorg);
        return $codigo;
    }


    /**
     * Exclui Atividade.
     *
     * @param integer $nrorg       N?mero da Organiza??o.
     * @param integer $nratividade N?mero da Atividade.
     *
     * @return Ambigous <number, number>
     */
    public static function excluiAtividade($nrorg, $nratividade){
        $dado = GepAtividadeEntity::excluiAtividade($nrorg, $nratividade);
        return $dado;
    }


    /**
     * Consulta dados de uma atividade.
     *
     * @param integer $nrorg       N?mero da Organiza??o.
     * @param integer $nratividade N?mero da Atividade.
     *
     * @return Ambigous <<type>, multitype:, mixed>|boolean
     */
    public static function retornaDadosAtividade($nrorg, $nratividade){
        try {
            $dado = GepAtividadeEntity::retornaDadosAtividade($nrorg, $nratividade);
            return $dado;
        } catch (Exception $e){
            switch ($e->getCode()){
            case GepAtividadeEntity::SEM_REGISTRO:
                return false;
            default:
                return tk_message('Erro desconhecido'.$e->getMessage());
            }
        }

    }


    /**
     * Retorna a consulta de atividades.
     *
     * @param  integer $nrorg         Numero da organizacao.
     * @param  integer $nrparcnegocio Numero do parceiro de negocio.
     *
     * @return string  $dados       Dados da consulta.
     */
    public static function retornaAtividades($nrorg, $nrparcnegocio){
        $dados = GepAtividadeEntity::retornaAtividades($nrorg, $nrparcnegocio);
        return $dados;
    }


    /**
     * Altera Status da atividade.
     *
     * @param string  $upidsituacaotiv Situa??o da Atividade.
     * @param integer $nrorg           N?mero da Organiza??o.
     * @param integer $nratividade     N?mero da Atividade.
     *
     * @return Ambigous <number, number>
     */
    public static function alteraStatusAtividade($upidsituacaotiv, $nrorg, $nratividade){
        $dado = GepAtividadeEntity::alteraStatusAtvidade($upidsituacaotiv, $nrorg, $nratividade);

        return $dado;
    }


    /**
     * Exporta itens de trabalho (metadados e arquivos) da atividade informada para o caminho especificado.
     *
     * @static
     * @throws Exception
     * @param  $nratividade
     * @param null $path
     * @return array
     */
    public static function exportaItemTrabalhoAtiv($nratividade, $path = null, $executa = true) {

        $metadata_export = array();
        $file_export = array();
        $return = array(    ManipulacaoArquivoLogic::ITEM_ARQUIVO   =>  array(),
                            ManipulacaoArquivoLogic::ITEM_METADATA   =>  array(),);

        if (empty($nratividade) === true) {
            throw new Exception("Atividade inv?da, n?foi poss?l exportar itens.");
        }

        $nrorg = Ambiente::getNrorg();
        $itens = GepItemtrabalhoativ::getObjectsByWhere("NRORG = ? AND NRATIVIDADE = ?", array($nrorg, $nratividade));

        foreach ($itens as $item) {
            $dsitemtrabalho = $item->get('DSITEMTRABALHO');

            if (empty($dsitemtrabalho) !== true) {
                $metadata_export[] = $dsitemtrabalho;
            } else {
                // fetch files to export
                $nritemtrabalho = $item->get('NRITEMTRABALHO');

                if (empty($nritemtrabalho) !== true) {
                    $itemtrabalho = GepItemtrabalho::getObjectByWhere("NRORG = ? AND NRITEMTRABALHO = ?", array($nrorg, $nritemtrabalho));

                    if (($itemtrabalho instanceof GepItemtrabalhoEntity) === true) {
                        $file_export[] = $itemtrabalho->toArray();
                    }
                }
            }
        }

        $nmacesso = Ambiente::getNmacesso();
        $conn = ManipulacaoArquivoLogic::getConnectionByNmacesso($nmacesso);

        // caminhos
        $path = $path !== null ? $path : ManipulacaoArquivoLogic::retornaCaminhoRepoAtividade($nratividade);
        $path = TkFile::mkdir($path);

        if (count($metadata_export) > 0) {
            // export metadata
            foreach ($metadata_export as $object_name) {
                try {
                    $caminho = $path.$object_name.'.zip';

                    if ($executa === true) {
                        $file = ManipulacaoArquivoLogic::executametadiffExport($object_name, $caminho, $conn);
                        $file = new TkFile(realpath($file->getFullpath()));

                        if ($file instanceof TkFile) {
                            tk_warning('Metadados de item exportado com sucesso: '.$object_name);
                            $return[ManipulacaoArquivoLogic::ITEM_METADATA][] = $file;
                        }
                    } else {
                        if (file_exists($caminho)) {
                            $file = new TkFile($caminho);
                            if ($file instanceof TkFile) {
                                $return[ManipulacaoArquivoLogic::ITEM_METADATA][] = $file;
                            }
                        } else {
                            throw new Exception('Item de trabalho n?encontrado: "'.$caminho.'".');
                        }
                    }
                } catch (Exception $e) {
                    tk_warning('Erro ao exportar item do metadados "'.$object_name.'": '.$e->getMessage());
                    continue;
                }
            }
        }

        if (count($file_export) > 0) {
            // export files
            foreach ($file_export as $itemtrabalho) {

                try {
                    $caminho_remoto = ApropriacaoAtividadelogic::retornaUrlRemotaItemTrabalho($itemtrabalho);

                    $nrarvorepai = $itemtrabalho['NRARVOREPAI'];
                    $arvore = ApropriacaoAtividadelogic::retornaCaminhoItemDeTrabalho($nrarvorepai, ApropriacaoAtividadelogic::retornaNomeDirVersao());
                    $primeiro_item_arvore = reset(explode('/', $arvore));

                    $caminho_local = $path.$arvore.'/'.$itemtrabalho['NMITEMTRABALHO'];

                    if ($executa === true) {

                        $file = ApropriacaoAtividadelogic::buscaSalvaArquivoRemoto($caminho_remoto, $caminho_local);

                        if ($file instanceof TkFile) {
                            // zip
                            $zip_file_name = reset(explode('.',$itemtrabalho['NMITEMTRABALHO']));
                            $zip_file_path = realpath($path).DIRECTORY_SEPARATOR.$zip_file_name.'.zip';
                            $zip = new TkFile($zip_file_path, true, TkFile::MODE_ZIP);
                            $zip->add(realpath($path).DIRECTORY_SEPARATOR.$primeiro_item_arvore.DIRECTORY_SEPARATOR);
                            $zip->save();

                            if (file_exists($zip->getFullpath())) {
                                $dir = new TkFile(realpath($path).DIRECTORY_SEPARATOR.$primeiro_item_arvore.DIRECTORY_SEPARATOR, false, TkFile::MODE_DIR);
                                $dir->destroy();
                            }

                            tk_warning('Arquivo exportado com sucesso: '.realpath($zip->getFullpath()));
                            $return[ManipulacaoArquivoLogic::ITEM_ARQUIVO][] = $zip;
                        } else {
                            throw new Exception('Houve um erro inesperado ao exportar item de trabalho ('.$caminho_remoto.').');
                        }
                    } else {

                        $zip_file_name = reset(explode('.',$itemtrabalho['NMITEMTRABALHO']));
                        $zip_file_path = $path.DIRECTORY_SEPARATOR.$zip_file_name.'.zip';

                        if (file_exists($zip_file_path)) {
                            $file = new TkFile($zip_file_path);
                            if ($file instanceof TkFile) {
                                $return[ManipulacaoArquivoLogic::ITEM_ARQUIVO][] = $file;
                            }
                        } else {
                            throw new Exception('Item de trabalho n?encontrado: "'.$caminho.'".');
                        }
                    }

                } catch (Exception $e) {
                    tk_warning('Erro ao buscar e salvar o arquivo "'.$itemtrabalho['NMITEMTRABALHO'].'": '.$e->getMessage());
                    continue;
                }
            }
        }

        return $return;
    }


    /**
     * Se houver atividade e observa? do operador ele atualiza o registro se nao salva.
     *
     * @param integer  $nratividade       Numero da atividade.
     * @param integer  $nrorg             Numero da organiza??o.
     * @param integer  $nratividadeproc   Numero do operador da observacao.
     * @param integer  $nrsolicitacao     Numero da solicita??o.
     * @param integer  $nrparcnegosoli    Numero do parceiro de neg?cio.
     * @param integer  $nrparcopersoli    Numero do operador solicitante.
     * @param integer  $nrparcoperresp    Numero do operador respons?vel.
     * @param integer  $nrparcoperconc    Numero do operador conclusor.
     * @param integer  $dsreferenciaativ  Refer?ncia da atividade.
     * @param string   $idsituacaoativ    Situa??o da atividade.
     * @param dateTime $dtinicioplanejado Data de inicio planejado.
     * @param dateTime $dtfimplanejado    Data de fim planejado.
     * @param float    $qthorasplanejada  Quantidade de horas planejado.
     * @param dateTime $dtfinalizacao     Data de finaliza??o.
     * @param dateTime $dtconclusao       Data de conclus?o.
     * @param string   $idativo           Atividade ativa.
     * @param string   $txobservatividade Observa??es da atividade.
     * @param date     $data			  Data de insercao.
     * @param integer  $cdoperador        Codigo do operador.
     * @param string   $txatividade       Texto da atividade.
     * @param string   $txobservacao      Texto de observa?.
     * @param integer  $nrparcnegoobserv  Numero do parceiro de negocio da observacao.
     *
     * @return boolean
     */
    public static function salvaAtividade(
        $nratividade,
        $nrorg,
        $nratividadeproc,
        $nrsolicitacao,
        $nrparcnegosoli,
        $nrparcopersoli,
        $nrparcoperresp,
        $nrparcoperconc,
        $dsreferenciaativ,
        $idsituacaoativ,
        $dtinicioplanejado,
        $dtfimplanejado,
        $qthorasplanejada,
        $dtfinalizacao,
        $dtconclusao,
        $idativo,
        $txobservatividade,
        $data,
        $cdoperador,
        $txatividade,
        $txobservacao,
        $nrparcnegoobserv
    ){
        $registro = self::retornaDadosAtividade($nrorg, $nratividade);

        if ($registro === false) {
             GepAtividadeEntity::incluiAtividade(
                 $nratividade,
                 $nrorg,
                 $nratividadeproc,
                 $nrsolicitacao,
                 $nrparcnegosoli,
                 $nrparcopersoli,
                 $nrparcoperresp,
                 $nrparcoperconc,
                 $dsreferenciaativ,
                 $idsituacaoativ,
                 $dtinicioplanejado,
                 $dtfimplanejado,
                 $qthorasplanejada,
                 $dtfinalizacao,
                 $dtconclusao,
                 $idativo,
                 $txobservatividade,
                 $data,
                 $cdoperador,
                 $txatividade
             );

        } else {
              GepAtividadeEntity::alteraAtividade(
                  $nratividade,
                  $nrorg,
                  $nratividadeproc,
                  $nrsolicitacao,
                  $nrparcnegosoli,
                  $nrparcopersoli,
                  $nrparcoperresp,
                  $nrparcoperconc,
                  $dsreferenciaativ,
                  $idsituacaoativ,
                  $dtinicioplanejado,
                  $dtfimplanejado,
                  $qthorasplanejada,
                  $dtfinalizacao,
                  $dtconclusao,
                  $idativo,
                  $txobservatividade,
                  $data,
                  $cdoperador,
                  $txatividade
              );
        }

        if (empty($txobservacao) === false) {
            GepObservacaoativLogic::salvaObservacaoOperador(
                $nratividade,
                $nrparcnegoobserv,
                $txobservacao,
                $nrorg,
                $cdoperador,
                $data
            );
        }

        return true;
    }


    /**
     * Retorna objeto da atividade.
     *
     * @static
     * @throws Exception
     * @param string|int $nratividade
     * @param string|int $nrorg
     * @return GepAtividadeEntity
     */
    public static function retornaAtividade($nratividade, $nrorg = null) {

        $nrorg = $nrorg === null ? Ambiente::getNrorg() : $nrorg;
        $atividade = GepAtividade::getObjectByWhere('NRATIVIDADE = ? AND NRORG = ?', array($nratividade, $nrorg));

        if (($atividade instanceof GepAtividadeEntity) !== true) {
            throw new Exception('Atividade inv?da.');
        }

        return $atividade;
    }


    /**
     * Metodo para verificar se existe atividade apropriada.
     *
     * @param integer $nrorg     Numero da organizacao.
     * @param integer $atividade Numero da atividade.
     *
     * @return GepConvocadoreuniaoEntity|boolean
     */

    public function verificarAtividade($nrorg, $atividade){
        try{
            $dados = $this->_GepConvocadoreuniaoEntity->verificarAtividadeGerada($nrorg, $atividade);
            return $dados;
        }
        catch (Exception $e){
            switch ($e->getCode()){
            case GepAtividadeEntity::SEM_REGISTRO:
                    return false;
            default:
                    tk_warning('Erro desconhecido: '.$e->getMessage());
            }
        }
    }


}